{% extends "registration/base_registration.html" %}
{% load i18n %}

{% block card_header%}
  <h2> Password Reset Started </h2>
{% endblock %}

{% block card_body %}

  <p>{% trans "We've emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly." %}</p>

  <p>{% trans "If you don't receive an email, please make sure you've entered the address you registered with, and check your spam folder." %}</p>
{% endblock %}
