{% extends 'registration/base_registration.html' %}

{% block card_header %}
  <h3>Email Confirmation Required</h3>
{% endblock %}
{% block card_body %}
  <p>To continue with login check your email.</p>
  <p>You may need to check your spam or junk folder.</p>

{% endblock %}
