{% extends 'registration/base_registration.html' %}
{% load i18n %}

{% block card_header%}
  <h2> Password Reset </h2>
{% endblock %}

{% block card_body %}

    <p>{% trans "Forgotten your password? Enter your email address below, and we'll email instructions for setting a new one." %}</p>
    <form method="post">
      {% csrf_token %}

      {% include 'registration/form.html' %}

      <input type="submit" value="Reset My Password" class="btn btn-primary my-2" />
    </form>

{% endblock %}
