# Generated by Django 4.1.4 on 2023-02-27 00:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0007_auto_20230227_0020'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='loginrule',
            name='jit_creation',
        ),
        migrations.AlterField(
            model_name='loginrule',
            name='jit_creation_type',
            field=models.CharField(blank=True, choices=[('attributes', 'Matching Attributes'), ('idp', 'Matching Attributes or IdP login')], help_text='If enabled a user account will be automatically created if one does not exist.', max_length=10, verbose_name='Just in time account creation type'),
        ),
    ]
