# Generated by Django 4.0.6 on 2022-10-04 22:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0003_loginrule_jit_username_format'),
    ]

    operations = [
        migrations.RenameField(
            model_name='loginrule',
            old_name='jit_attributes',
            new_name='saved_attributes',
        ),
        migrations.AlterField(
            model_name='loginrule',
            name='saved_attributes',
            field=models.JSONField(blank=True, default=dict,
                                   help_text="Attributes to set on user instance when creating user. This is a Django template string that supports lookups like {{profile.raw_attributes['some-extended-attribute']}}. <a target='_blank' href='https://workos.com/docs/reference/sso/profile'>Profile</a> is defined by WorkOS"),
        ),
    ]
