# Generated by Django 4.0.6 on 2022-10-05 06:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0004_rename_jit_attributes_loginrule_saved_attributes'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='loginrule',
            name='jit_username_format',
        ),
        migrations.AddField(
            model_name='userlogin',
            name='rule',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='workos_login.loginrule'),
        ),
    ]
