{% extends "registration/base_registration.html" %}
{% load i18n %}

{% block card_header%}
  <h2> Password Reset </h2>
{% endblock %}

{% block card_body %}


  {% if validlink %}
    <p>{% trans "Please enter your new password twice so we can verify you typed it in correctly." %}</p>
    <form method="post">
      {% csrf_token %}
      {% include 'registration/form.html' %}

      <input type="submit" value="Change My Password" class="btn btn-primary my-2" />
    </form>
  {% else %}
    <p>{% trans "The password reset link was invalid, possibly because it has already been used.  Please request a new password reset." %}</p>
  {% endif %}


{% endblock %}
