# Generated by Django 4.0.6 on 2022-09-20 04:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0002_auto_20220803_2242'),
    ]

    operations = [
        migrations.AddField(
            model_name='loginrule',
            name='jit_username_format',
            field=models.CharField(blank=True, choices=[('email', 'Email Address'), ('idp', 'Identity Provider ID'), ('workos', 'Unique ID')], help_text='Username format for created accounts', max_length=10),
        ),
    ]
