{% extends 'registration/base_registration.html' %}

{% block card_header%}
  <h2> Password Reset Successful </h2>
{% endblock %}

{% block card_body %}

  <p>Your password has been set.  You may go ahead and log in now.</p>

  <p><a class="btn btn-primary" href="{{ login_url }}">Log in now</a></p>

{% endblock %}
