# Generated by Django 4.1.4 on 2023-02-27 00:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0005_remove_loginrule_jit_username_format_userlogin_rule'),
    ]

    operations = [
        migrations.AddField(
            model_name='loginrule',
            name='jit_creation_type',
            field=models.CharField(blank=True, choices=[('attributes', 'Matching Attributes'), ('idp', 'Matching Attributes or IdP login')], max_length=10),
        ),
    ]
