# Generated by Django 4.1.4 on 2023-02-27 00:20

from django.db import migrations


def forward(apps, schema_editor):
    LoginRule = apps.get_model('workos_login', 'LoginRule')
    LoginRule.objects.filter(jit_creation=True).update(jit_creation_type="attributes")

class Migration(migrations.Migration):

    dependencies = [
        ('workos_login', '0006_loginrule_jit_creation_type'),
    ]

    operations = [
        migrations.RunPython(forward, elidable=True),
    ]
