
{% if form.non_field_errors %}
  <div class="alert alert-danger">
    {{ form.non_field_errors }}
  </div>
{% endif %}

{% for field in form.visible_fields %}


  <div class="form-row">
    {{ field.label_tag }} {{ field }}
    <div class="invalid-feedback">
      {{ field.errors }}
    </div>
    {% if field.help_text %}
      <small class="text-muted">{{ field.help_text|safe }}</small>
    {% endif %}
  </div>

{% endfor %}
{% for hidden in form.hidden_fields %}
  {{ hidden }}
{% endfor %}
